import react, {
  cloneElement,
  JSXElementConstructor,
  ReactElement,
  useEffect,
  useRef,
  useState,
} from 'react';
import TileSet from './TileSet';
import { Point, Rectangle, TileKeyObject } from './types';
import { CoordinateSystem } from './LiveMap';

export interface TiledLayerProperties {
  /**
   * A key identifying the top left tile
   */
  keyObject: TileKeyObject;
  /**
   * The translation to apply.
   */
  shift: Point;
  /**
   * The zoom to apply. If equal to 256 (the tile size), the layer is considered active and should add tiles which are in its viewport
   */
  zoom: number;
  /**
   * The coordinate system
   */
  coordinateSystem: CoordinateSystem;
  /** Slippy graphics are non scalable SVG snippets defined by geo locations */
  slippyGraphics?: ReactElement<any, string | JSXElementConstructor<any>>[];
}

/**
 *
 * @param props
 * @returns A layer of tiles.
 * This component wraps a `<TileSet>` in an SVG `<g>` element taking care of the scale and translation.
 *
 */
export const TiledLayer: react.FC<TiledLayerProperties> = (
  props: TiledLayerProperties
) => {
  const g = useRef<SVGGElement>(null);

  const [viewPort, setViewPort] = useState<Rectangle>();

  useEffect(() => {
    console.log(
      `TiledLayer zoom: ${props.zoom}, useEffect, g.current: ${g.current}`
    );
    if (
      props.zoom === 256 &&
      g.current !== null &&
      g.current.ownerSVGElement !== null &&
      g.current.ownerSVGElement.parentElement !== null
    ) {
      const nearerHTMLParent = g.current.ownerSVGElement.parentElement;
      setViewPort({
        topLeft: {
          x:
            props.keyObject.x +
            Math.floor(
              -props.coordinateSystem.shift.x /
                props.coordinateSystem.zoom /
                256
            ),
          y:
            props.keyObject.y +
            Math.floor(
              -props.coordinateSystem.shift.y /
                props.coordinateSystem.zoom /
                256
            ),
        },
        bottomRight: {
          x:
            props.keyObject.x +
            Math.ceil(
              (-props.coordinateSystem.shift.x + nearerHTMLParent.offsetWidth) /
                props.coordinateSystem.zoom /
                256
            ) -
            1,
          y:
            props.keyObject.y +
            Math.ceil(
              (-props.coordinateSystem.shift.y +
                nearerHTMLParent.offsetHeight) /
                props.coordinateSystem.zoom /
                256
            ) -
            1,
        },
      });
    }
  }, [props]);

  console.log(
    `Rendering TiledLayer, zoom: ${props.zoom}, viewPort: ${JSON.stringify(
      viewPort
    )}, slippyGraphics: ${props.slippyGraphics}`
  );
  return (
    <>
      <g
        transform={`scale(${props.zoom}) translate(${props.shift.x}, ${props.shift.y})`}
        ref={g}
      >
        <TileSet
          keyObject={{
            provider: props.keyObject.provider,
            zoomLevel: props.keyObject.zoomLevel,
            x: 0,
            y: 0,
          }}
          viewPort={viewPort}
        />
        {props.slippyGraphics !== undefined && viewPort !== undefined ? (
          // Slippy graphics (if needed)
          <>
            {props.slippyGraphics.map((slippyGraphic) =>
              cloneElement(slippyGraphic, {
                keyObject: props.keyObject,
                zoom: props.coordinateSystem.zoom,
                viewPort: viewPort,
              })
            )}
            {console.log('TiledLayer: adding slippyGraphics')},
          </>
        ) : null}
      </g>
    </>
  );
};

export default TiledLayer;
